% Kernel learning via quadratically constrained quadratical programming (QCQP) in multi-class case
% Coded by Shuiwang Ji (shuiwang.j@asu.edu, Arizona State University)
% Modified by Jianhui Chen (jianhui.chen@asu.edu, Arizona State University)

%                Input Parameters
% "G" : the cell containing centered kernel matrix (trace value = 1) 
% "lambda" : regularization parameter
% "a" : the indicator vector

%               Output parameters
% "theta" : the coefficients of kernel matrices
% "time" : computation time in seconds

function [theta, time, beta, r_value, res] = QCQP_wo_tau_multi_msk_new(G, H, lambda)

time0 = cputime;

[ n k ] = size(H);
p = length(G);

H_stack = [];
for i = 1:k
    H_stack = [H_stack; H(:, i)];
end

for i = 1:p
    G_bdiag{i} = sparse(n*k, n*k);
    for j = 1:k
        G_bdiag{i}((j-1)*n+1:j*n, (j-1)*n+1:j*n) = G{i};
    end
end

clear prob;

prob.c = [H_stack; -1/(4*lambda)];

prob.qosubi = 1:n*k+1;
prob.qosubj = 1:n*k+1;
prob.qoval = [-1/2*ones(n*k, 1); 0];

prob.qcsubk = [];
prob.qcsubi = [];
prob.qcsubj = [];
prob.qcval = [];
for i = 1:p
    [row_num, col_num, entry_val] = find(tril(G_bdiag{i}));
    prob.qcsubk = [prob.qcsubk; i*ones(length(entry_val), 1)];
    prob.qcsubi = [prob.qcsubi; row_num];
    prob.qcsubj = [prob.qcsubj; col_num];
    prob.qcval = [prob.qcval; 2 .* entry_val];
end

prob.a = [sparse(p, n*k) -1*ones(p,1)];

prob.blc = [-inf*ones(p, 1)];

prob.buc = zeros(p, 1);

[r_value, res] = mosekopt('maximize info',prob);

beta_stack = res.sol.itr.xx(1:n*k, 1);
beta = [];
for i = 1:k
   beta = [beta, beta_stack((i-1)*n+1:i*n)]; 
end

theta = res.sol.itr.y .* (4 * lambda);

time = cputime - time0;








